﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using AutoShutdownDotNet.Actions;
using AutoShutdownDotNet.Models;

namespace AutoShutdownDotNet
{
    public partial class DoTaskForm : Form
    {
        #region Properties

        public Task Task { get; set; }

        public string Title
        {
            get { return lblTitle.Text; }
            set { this.Text = lblTitle.Text = value; }
        }

        public int TaskId { get; set; }

        public void Stop()
        {
            timer.Enabled = false;
        }

        private AutoShutdownAction _action;
        public AutoShutdownAction Action 
        { 
            get { return _action; } 
            set 
            { 
                _action = value;
                switch (value)
                {
                    case AutoShutdownAction.Hibernate:
                        if (timer.Enabled)
                            lblDescription.Text = "Your Computer will be Hibernated in a few seconds.";
                        else
                            lblDescription.Text = "Your Computer will be Hibernated, what are you going to do?";
                        pictureBox.Image = Resources.Icons.hibernate48;
                        break;
                    case AutoShutdownAction.Lock:
                        if (timer.Enabled)
                            lblDescription.Text = "Your Computer will be Locked in a few seconds.";
                        else
                            lblDescription.Text = "Your Computer will be Locked, what are you going to do?";
                        pictureBox.Image = Resources.Icons.Lock48;
                        break;
                    case AutoShutdownAction.Logoff:
                        if(timer.Enabled)
                            lblDescription.Text = "You will be Logged off in a few seconds.";
                        else
                            lblDescription.Text = "Your Computer will be Logged off, what are you going to do?";
                        pictureBox.Image = Resources.Icons.logoff48;
                        break;
                    case AutoShutdownAction.PowerOff:
                        if(timer.Enabled)
                            lblDescription.Text = "Your Computer will be Shutdown in a few seconds.";
                        else
                            lblDescription.Text = "Your Computer will be Shutdown, what are you going to do?";
                        pictureBox.Image = Resources.Icons.shutdown48;
                        break;
                    case AutoShutdownAction.Reboot:
                        if(timer.Enabled)
                            lblDescription.Text = "Your Computer will be Reboot in a few seconds.";
                        else
                            lblDescription.Text = "Your Computer will be Rebbot, what are you going to do?";
                        pictureBox.Image = Resources.Icons.reboot48;
                        break;
                    case AutoShutdownAction.RunApplication:
                        if(timer.Enabled)
                            lblDescription.Text = "An application will be executed in a few seconds." + Environment.NewLine +
                                              "Path: " + FilePath;
                        else
                            lblDescription.Text = "An application will be executed, what are you going to do?" + Environment.NewLine +
                                              "Path: " + FilePath;
                        pictureBox.Image = Resources.Icons.run48;
                        break;
                    case AutoShutdownAction.Standby:
                        if(timer.Enabled)
                            lblDescription.Text = "Your Computer will be put to Sleep in a few seconds.";
                        else
                            lblDescription.Text = "Your Computer will be put to Sleep, what are you going to do?";
                        pictureBox.Image = Resources.Icons.sleep48;
                        break;
                }
            } 
        }

        public string FilePath { get; set; }

        public int _timer = 0;
        public int Timer
        {
            get { return _timer; }
            set { _timer = value;
                lblTimer.Text = value.ToString() + " second(s) to decide.";
                if(value == 0 && timer.Enabled)
                {
                    DoNow();
                }
                if (value < 10)
                    lblTimer.ForeColor = Color.Red;
            }
        }

        #endregion


        public DoTaskForm(Task task)
        {
            InitializeComponent();
            Task = task;
            timer.Enabled = (task.Timer > 0);
            Timer = task.Timer;
            lblTimer.Visible = (task.Timer > 0);
            btnOK.Visible = (task.Timer > 0);
            Title = task.Title;
            Action = task.Action;
            FilePath = Task.FilePath;
            TaskId = task.Id;
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            Timer--;
        }

        private void DoNow()
        {
            DoAction doAction = new DoAction(Task);
            doAction.Do();
            Close();
        }

        private void btnDoItNow_Click(object sender, EventArgs e)
        {
            DoNow();
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            Hide();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            Stop();
            History history = new History();
            history.Action = Task.Action;
            history.FilePath = Task.FilePath;
            history.Hour = Task.Hour;
            history.Minute = Task.Minute;
            history.When = DateTime.Now;
            history.Succeed = false;
            history.Comment = "User canceled the operation.";
            history.Insert();
            Close();
        }
    }
}
