﻿using System;
using System.Collections.Generic;
using System.Text;
using AutoShutdownDotNet.DataAccess;

namespace AutoShutdownDotNet.Models
{

    public enum AutoShutdownAction
    {
        PowerOff = 1,
        Reboot,
        Logoff,
        Standby,
        Hibernate,
        RunApplication,
        ShowMessage,
        Lock
    }

    public class Task
    {
        public int Id { get; set; }
        public string Title { get; set; }
        public DateTime? When { get; set; }
        public int Hour { get; set; }
        public int Minute { get; set; }
        public bool AskBefore { get; set; }
        public AutoShutdownAction Action { get; set; }
        public int Timer { get; set; }
        public bool Active { get; set; }
        public string Message { get; set; }
        public string FilePath { get; set; }
        public string Comment { get; set; }
        public DateTime CreatedDate { get; set; }
        public DateTime LastModified { get; set; }

        public string Time
        {
            get
            {
                string h = Hour.ToString();
                if (h.Length == 1)
                    h = "0" + h;
                string m = Minute.ToString();
                if (m.Length == 1)
                    m = "0" + m;
                return h + ":" + m;
            }
        }

        public string WhenToString()
        {
            if (When.HasValue)
                return When.Value.ToShortDateString();
            return "Everyday";
        }

        public static string WhenToString(DateTime? when)
        {
            if(when.HasValue)
                return when.Value.ToShortDateString();
            return "Everyday";
        }

        public static string TimeToString(int hour, int minute)
        {
            string h = hour.ToString();
            if (h.Length == 1)
                h = "0" + h;
            string m = minute.ToString();
            if (m.Length == 1)
                m = "0" + m;
            return h + ":" + m;
        }

        public static string ActionToString(AutoShutdownAction action)
        {
            switch (action)
            {
                case AutoShutdownAction.Hibernate:
                    return "Hibernate";
                case AutoShutdownAction.Logoff:
                    return "Logoff";
                case AutoShutdownAction.PowerOff:
                    return "PowerOff";
                case AutoShutdownAction.Reboot:
                    return "Reboot";
                case AutoShutdownAction.RunApplication:
                    return "Run Application";
                case AutoShutdownAction.ShowMessage:
                    return "Show Message";
                case AutoShutdownAction.Standby:
                    return "Standby";
                case AutoShutdownAction.Lock:
                    return "Lock";
                default:
                    return "Undefined";
            }
        }

        public void Insert()
        {
            CreatedDate = DateTime.Now;
            Database db = new Database();
            db.Insert(this);
            Id = db.GetLastTaskId();
        }

        public void Update()
        {
            if(Id > 0)
            {
                LastModified = DateTime.Now;
                Database db = new Database();
                db.Update(this);
            }
            else
            {
                throw new Exception("You need to pass a valid TaskId for update this object.");
            }
        }

        public void Delete()
        {
            if (Id > 0)
            {
                Database db = new Database();
                db.Delete(this);
            }
            else
            {
                throw new Exception("You need to pass a valid TaskId for update this object.");
            }
        }

        public bool Exist()
        {
            Database db = new Database();
            return db.Exist(Id,When, Hour, Minute);
        }

        public static List<Task> Select(DateTime? from, DateTime? to)
        {
            return new Database().SelectAllTasks(from, to);
        }

        public static Task Load(int id)
        {
            return new Database().Load(id);
        }

        public static bool Exist(int taskId,DateTime? when, int hour, int minute)
        {
            Database db = new Database();
            return db.Exist(taskId,when, hour, minute);
        }
    }
}
